import 'package:flutter/material.dart';

class ScalableText extends StatelessWidget {
  final String text;
  final TextStyle? style;
  final TextAlign textAlign;
  final int? maxLines;
  final double minFontSize;
  final double maxFontSize;
  final bool allowFontScalingShrink;
  final bool allowFontScalingGrow;
  final TextOverflow overflow;
  final double widthFactor;

  const ScalableText(
      this.text, {
        Key? key,
        this.style,
        this.textAlign = TextAlign.center,
        this.maxLines,
        this.minFontSize = 8.0,
        this.maxFontSize = 100.0,
        this.allowFontScalingShrink = true,
        this.allowFontScalingGrow = true,
        this.overflow = TextOverflow.clip,
        this.widthFactor = 0.9, // Use 90% of available width by default
      }) : super(key: key);

  @override
  Widget build(BuildContext context) {
    return LayoutBuilder(
      builder: (BuildContext context, BoxConstraints constraints) {
        // Get available width and height
        final availableWidth = constraints.maxWidth * widthFactor;
        final availableHeight = constraints.maxHeight;

        // Base font size from style or default
        final defaultStyle = style ?? Theme.of(context).textTheme.bodyLarge;
        final baseFontSize = defaultStyle?.fontSize ?? 14.0;

        // Initial text style
        final baseStyle = defaultStyle?.copyWith(
          fontSize: baseFontSize,
        ) ?? TextStyle(fontSize: baseFontSize);

        // Calculate optimal text size
        double scaledFontSize = _calculateOptimalFontSize(
          text: text,
          maxLines: maxLines ?? 1,
          maxWidth: availableWidth,
          maxHeight: availableHeight,
          style: baseStyle,
          minFontSize: minFontSize,
          maxFontSize: maxFontSize,
          allowShrink: allowFontScalingShrink,
          allowGrow: allowFontScalingGrow,
        );

        // Apply the calculated font size to the style
        final scaledStyle = baseStyle.copyWith(
          fontSize: scaledFontSize,
          height: 1.0, // Important for accurate vertical sizing
        );

        return Container(
          width: double.infinity,
          height: double.infinity,
          alignment: Alignment.center,
          child: Text(
            text,
            style: scaledStyle,
            textAlign: textAlign,
            maxLines: maxLines,
            overflow: overflow,
          ),
        );
      },
    );
  }

  double _calculateOptimalFontSize({
    required String text,
    required int maxLines,
    required double maxWidth,
    required double maxHeight,
    required TextStyle style,
    required double minFontSize,
    required double maxFontSize,
    required bool allowShrink,
    required bool allowGrow,
  }) {
    // Start with the current font size
    double fontSize = style.fontSize ?? 14.0;

    // Function to check if text fits within constraints
    bool doesTextFit(double size) {
      final textPainter = TextPainter(
        text: TextSpan(
          text: text,
          style: style.copyWith(fontSize: size),
        ),
        textDirection: TextDirection.ltr,
        maxLines: maxLines,
        textAlign: textAlign,
      );

      textPainter.layout(maxWidth: maxWidth);

      return textPainter.width <= maxWidth &&
          textPainter.height <= maxHeight &&
          !textPainter.didExceedMaxLines;
    }

    // Binary search to find optimal font size
    double low = minFontSize;
    double high = maxFontSize;
    double optimal = fontSize;

    // If current size doesn't fit and we can shrink
    if (!doesTextFit(fontSize) && allowShrink) {
      high = fontSize;

      while (high - low > 0.5) {  // 0.5 is precision
        double mid = (low + high) / 2;
        if (doesTextFit(mid)) {
          low = mid;
          optimal = mid;
        } else {
          high = mid;
        }
      }

      return optimal;
    }
    // If current size fits and we can grow
    else if (doesTextFit(fontSize) && allowGrow) {
      low = fontSize;

      while (high - low > 0.5) {  // 0.5 is precision
        double mid = (low + high) / 2;
        if (doesTextFit(mid)) {
          low = mid;
          optimal = mid;
        } else {
          high = mid;
        }
      }

      return optimal;
    }

    // Return current size if no scaling needed or allowed
    return fontSize;
  }
}
